@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022, [Ribose Inc](https://www.ribose.com).
+ * Copyright (c) 2017-2024, [Ribose Inc](https://www.ribose.com).
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,13 +33,12 @@
 #include "config.h"
 
 #include <librepgp/stream-packet.h>
-#include "key_store_pgp.h"
 #include "key_store_g10.h"
 
 #include "crypto/common.h"
 #include "crypto/mem.h"
 #include "crypto/cipher.hpp"
-#include "pgp-key.h"
+#include "key.hpp"
 #include "time-utils.h"
 
 #include "g23_sexp.hpp"
@@ -250,7 +249,7 @@ lookup_var(const sexp_list_t *list, const std::string &name) noexcept
     // We are looking for a list element  (condition 1)
     // that:
     //  -- has at least two SEXP elements (condition 2)
-    //  -- has a SEXP string at 0 postion (condition 3)
+    //  -- has a SEXP string at 0 position (condition 3)
     //     matching given name            (condition 4)
     auto match = [name](const std::shared_ptr<sexp_object_t> &ptr) {
         bool r = false;
@@ -262,7 +261,7 @@ lookup_var(const sexp_list_t *list, const std::string &name) noexcept
         }
         return r;
     };
-    auto r3 = std::find_if(list->begin(), list->end(), match);
+    auto r3 = std::find_if(list->begin(), list->end(), std::move(match));
     if (r3 == list->end())
         RNP_LOG("Haven't got variable '%s'", name.c_str());
     else
@@ -287,7 +286,7 @@ lookup_var_data(const sexp_list_t *list, const std::string &name) noexcept
 }
 
 static bool
-read_mpi(const sexp_list_t *list, const std::string &name, pgp_mpi_t &val) noexcept
+read_mpi(const sexp_list_t *list, const std::string &name, pgp::mpi &val) noexcept
 {
     const sexp_string_t *data = lookup_var_data(list, name);
     if (!data) {
@@ -297,13 +296,15 @@ read_mpi(const sexp_list_t *list, const std::string &name, pgp_mpi_t &val) noexc
     /* strip leading zero */
     const auto &bytes = data->get_string();
     if ((bytes.size() > 1) && !bytes[0] && (bytes[1] & 0x80)) {
-        return mem2mpi(&val, bytes.data() + 1, bytes.size() - 1);
+        val.assign(bytes.data() + 1, bytes.size() - 1);
+    } else {
+        val.assign(bytes.data(), bytes.size());
     }
-    return mem2mpi(&val, bytes.data(), bytes.size());
+    return true;
 }
 
 static bool
-read_curve(const sexp_list_t *list, const std::string &name, pgp_ec_key_t &key) noexcept
+read_curve(const sexp_list_t *list, const std::string &name, pgp::ec::Key &key) noexcept
 {
     const sexp_string_t *data = lookup_var_data(list, name);
     if (!data) {
@@ -312,17 +313,17 @@ read_curve(const sexp_list_t *list, const std::string &name, pgp_ec_key_t &key)
 
     const auto &bytes = data->get_string();
     pgp_curve_t curve = static_cast<pgp_curve_t>(
-      id_str_pair::lookup(g10_curve_aliases, data->get_string(), PGP_CURVE_UNKNOWN));
+      id_str_pair::lookup(g10_curve_aliases, (const char *) bytes.data(), PGP_CURVE_UNKNOWN));
     if (curve != PGP_CURVE_UNKNOWN) {
         key.curve = curve;
         return true;
     }
-    RNP_LOG("Unknown curve: %.*s", (int) bytes.size(), (char *) bytes.data());
+    RNP_LOG("Unknown curve: %.*s", (int) bytes.size(), (const char *) bytes.data());
     return false;
 }
 
 void
-gnupg_sexp_t::add_mpi(const std::string &name, const pgp_mpi_t &mpi)
+gnupg_sexp_t::add_mpi(const std::string &name, const pgp::mpi &mpi)
 {
     auto sub_s_exp = add_sub();
     sub_s_exp->push_back(std::make_shared<sexp_string_t>(name));
@@ -330,27 +331,27 @@ gnupg_sexp_t::add_mpi(const std::string &name, const pgp_mpi_t &mpi)
     sub_s_exp->push_back(value_block);
 
     sexp_simple_string_t data;
-    size_t               len = mpi_bytes(&mpi);
+    size_t               len = mpi.size();
     size_t               idx;
 
-    for (idx = 0; (idx < len) && !mpi.mpi[idx]; idx++)
+    for (idx = 0; (idx < len) && !mpi[idx]; idx++)
         ;
 
     if (idx < len) {
-        if (mpi.mpi[idx] & 0x80) {
+        if (mpi[idx] & 0x80) {
             data.append(0);
-            data.std::basic_string<uint8_t>::append(mpi.mpi + idx, len - idx);
+            data.std::basic_string<uint8_t>::append(mpi.data() + idx, len - idx);
         } else {
-            data.assign(mpi.mpi + idx, mpi.mpi + len);
+            data.assign(mpi.data() + idx, mpi.data() + len);
         }
         value_block->set_string(data);
     }
 }
 
 void
-gnupg_sexp_t::add_curve(const std::string &name, const pgp_ec_key_t &key)
+gnupg_sexp_t::add_curve(const std::string &name, pgp_curve_t kcurve)
 {
-    const char *curve = id_str_pair::lookup(g10_curve_names, key.curve, NULL);
+    const char *curve = id_str_pair::lookup(g10_curve_names, kcurve, NULL);
     if (!curve) {
         RNP_LOG("unknown curve");
         throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
@@ -360,13 +361,13 @@ gnupg_sexp_t::add_curve(const std::string &name, const pgp_ec_key_t &key)
     psub_s_exp->add(name);
     psub_s_exp->add(curve);
 
-    if ((key.curve != PGP_CURVE_ED25519) && (key.curve != PGP_CURVE_25519)) {
+    if ((kcurve != PGP_CURVE_ED25519) && (kcurve != PGP_CURVE_25519)) {
         return;
     }
 
     psub_s_exp = add_sub();
     psub_s_exp->add("flags");
-    psub_s_exp->add((key.curve == PGP_CURVE_ED25519) ? "eddsa" : "djb-tweak");
+    psub_s_exp->add((kcurve == PGP_CURVE_ED25519) ? "eddsa" : "djb-tweak");
 }
 
 static bool
@@ -374,47 +375,50 @@ parse_pubkey(pgp_key_pkt_t &pubkey, const sexp_list_t *s_exp, pgp_pubkey_alg_t a
 {
     pubkey.version = PGP_V4;
     pubkey.alg = alg;
-    pubkey.material.alg = alg;
     switch (alg) {
-    case PGP_PKA_DSA:
-        if (!read_mpi(s_exp, "p", pubkey.material.dsa.p) ||
-            !read_mpi(s_exp, "q", pubkey.material.dsa.q) ||
-            !read_mpi(s_exp, "g", pubkey.material.dsa.g) ||
-            !read_mpi(s_exp, "y", pubkey.material.dsa.y)) {
+    case PGP_PKA_DSA: {
+        pgp::dsa::Key dsa;
+        if (!read_mpi(s_exp, "p", dsa.p) || !read_mpi(s_exp, "q", dsa.q) ||
+            !read_mpi(s_exp, "g", dsa.g) || !read_mpi(s_exp, "y", dsa.y)) {
             return false;
         }
+        pubkey.material = pgp::KeyMaterial::create(dsa);
         break;
-
+    }
     case PGP_PKA_RSA:
     case PGP_PKA_RSA_ENCRYPT_ONLY:
-    case PGP_PKA_RSA_SIGN_ONLY:
-        if (!read_mpi(s_exp, "n", pubkey.material.rsa.n) ||
-            !read_mpi(s_exp, "e", pubkey.material.rsa.e)) {
+    case PGP_PKA_RSA_SIGN_ONLY: {
+        pgp::rsa::Key rsa;
+        if (!read_mpi(s_exp, "n", rsa.n) || !read_mpi(s_exp, "e", rsa.e)) {
             return false;
         }
+        pubkey.material = pgp::KeyMaterial::create(alg, rsa);
         break;
-
+    }
     case PGP_PKA_ELGAMAL:
-    case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN:
-        if (!read_mpi(s_exp, "p", pubkey.material.eg.p) ||
-            !read_mpi(s_exp, "g", pubkey.material.eg.g) ||
-            !read_mpi(s_exp, "y", pubkey.material.eg.y)) {
+    case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN: {
+        pgp::eg::Key eg;
+        if (!read_mpi(s_exp, "p", eg.p) || !read_mpi(s_exp, "g", eg.g) ||
+            !read_mpi(s_exp, "y", eg.y)) {
             return false;
         }
+        pubkey.material = pgp::KeyMaterial::create(alg, eg);
         break;
+    }
     case PGP_PKA_ECDSA:
     case PGP_PKA_ECDH:
-    case PGP_PKA_EDDSA:
-        if (!read_curve(s_exp, "curve", pubkey.material.ec) ||
-            !read_mpi(s_exp, "q", pubkey.material.ec.p)) {
+    case PGP_PKA_EDDSA: {
+        pgp::ec::Key ec{};
+        if (!read_curve(s_exp, "curve", ec) || !read_mpi(s_exp, "q", ec.p)) {
             return false;
         }
-        if (pubkey.material.ec.curve == PGP_CURVE_ED25519) {
+        if (ec.curve == PGP_CURVE_ED25519) {
             /* need to adjust it here since 'ecc' key type defaults to ECDSA */
             pubkey.alg = PGP_PKA_EDDSA;
-            pubkey.material.alg = PGP_PKA_EDDSA;
         }
+        pubkey.material = pgp::KeyMaterial::create(pubkey.alg, ec);
         break;
+    }
     default:
         RNP_LOG("Unsupported public key algorithm: %d", (int) alg);
         return false;
@@ -424,44 +428,62 @@ parse_pubkey(pgp_key_pkt_t &pubkey, const sexp_list_t *s_exp, pgp_pubkey_alg_t a
 }
 
 static bool
-parse_seckey(pgp_key_pkt_t &seckey, const sexp_list_t *s_exp, pgp_pubkey_alg_t alg)
+parse_seckey(pgp_key_pkt_t &seckey, const sexp_list_t *s_exp)
 {
-    switch (alg) {
-    case PGP_PKA_DSA:
-        if (!read_mpi(s_exp, "x", seckey.material.dsa.x)) {
+    switch (seckey.alg) {
+    case PGP_PKA_DSA: {
+        pgp::mpi x;
+        auto     key = dynamic_cast<pgp::DSAKeyMaterial *>(seckey.material.get());
+        if (!key || !read_mpi(s_exp, "x", x)) {
             return false;
         }
-        break;
+        key->set_secret(x);
+        x.forget();
+        return true;
+    }
     case PGP_PKA_RSA:
     case PGP_PKA_RSA_ENCRYPT_ONLY:
-    case PGP_PKA_RSA_SIGN_ONLY:
-        if (!read_mpi(s_exp, "d", seckey.material.rsa.d) ||
-            !read_mpi(s_exp, "p", seckey.material.rsa.p) ||
-            !read_mpi(s_exp, "q", seckey.material.rsa.q) ||
-            !read_mpi(s_exp, "u", seckey.material.rsa.u)) {
+    case PGP_PKA_RSA_SIGN_ONLY: {
+        pgp::mpi d, p, q, u;
+        auto     key = dynamic_cast<pgp::RSAKeyMaterial *>(seckey.material.get());
+        if (!key || !read_mpi(s_exp, "d", d) || !read_mpi(s_exp, "p", p) ||
+            !read_mpi(s_exp, "q", q) || !read_mpi(s_exp, "u", u)) {
             return false;
         }
-        break;
+        key->set_secret(d, p, q, u);
+        d.forget();
+        p.forget();
+        q.forget();
+        u.forget();
+        return true;
+    }
     case PGP_PKA_ELGAMAL:
-    case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN:
-        if (!read_mpi(s_exp, "x", seckey.material.eg.x)) {
+    case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN: {
+        pgp::mpi x;
+        auto     key = dynamic_cast<pgp::EGKeyMaterial *>(seckey.material.get());
+        if (!key || !read_mpi(s_exp, "x", x)) {
             return false;
         }
-        break;
+        key->set_secret(x);
+        x.forget();
+        return true;
+    }
     case PGP_PKA_ECDSA:
     case PGP_PKA_ECDH:
-    case PGP_PKA_EDDSA:
-        if (!read_mpi(s_exp, "d", seckey.material.ec.x)) {
+    case PGP_PKA_EDDSA: {
+        pgp::mpi x;
+        auto     key = dynamic_cast<pgp::ECKeyMaterial *>(seckey.material.get());
+        if (!key || !read_mpi(s_exp, "d", x)) {
             return false;
         }
-        break;
+        key->set_secret(x);
+        x.forget();
+        return true;
+    }
     default:
-        RNP_LOG("Unsupported public key algorithm: %d", (int) alg);
+        RNP_LOG("Unsupported public key algorithm: %d", (int) seckey.alg);
         return false;
     }
-
-    seckey.material.secret = true;
-    return true;
 }
 
 static bool
@@ -495,7 +517,7 @@ decrypt_protected_section(const sexp_simple_string_t &encrypted_data,
     // find the protection format in our table
     info = find_format(prot.symm_alg, prot.cipher_mode, prot.s2k.hash_alg);
     if (!info) {
-        RNP_LOG("Unsupported format, alg: %d, chiper_mode: %d, hash: %d",
+        RNP_LOG("Unsupported format, alg: %d, cipher_mode: %d, hash: %d",
                 prot.symm_alg,
                 prot.cipher_mode,
                 prot.s2k.hash_alg);
@@ -515,9 +537,11 @@ decrypt_protected_section(const sexp_simple_string_t &encrypted_data,
 
     // decrypt
     decrypted_data = (uint8_t *) malloc(encrypted_data.size());
-    if (decrypted_data == NULL) {
+    if (!decrypted_data) {
+        /* LCOV_EXCL_START */
         RNP_LOG("can't allocate memory");
         goto done;
+        /* LCOV_EXCL_END */
     }
     dec = Cipher::decryption(
       info->cipher, info->cipher_mode, info->tag_length, info->disable_padding);
@@ -653,7 +677,7 @@ parse_protected_seckey(pgp_key_pkt_t &seckey, const sexp_list_t *list, const cha
 
     // we're all done if no password was provided (decryption not requested)
     if (!password) {
-        seckey.material.secret = false;
+        seckey.material->clear_secret();
         return true;
     }
 
@@ -704,7 +728,7 @@ parse_protected_seckey(pgp_key_pkt_t &seckey, const sexp_list_t *list, const cha
                protected_at_data->get_string().size());
     }
     // parse MPIs
-    if (!parse_seckey(seckey, decrypted_s_exp.sexp_list_at(0), seckey.alg)) {
+    if (!parse_seckey(seckey, decrypted_s_exp.sexp_list_at(0))) {
         RNP_LOG("failed to parse seckey");
         return false;
     }
@@ -743,20 +767,19 @@ parse_protected_seckey(pgp_key_pkt_t &seckey, const sexp_list_t *list, const cha
             return false;
         }
     }
-    seckey.material.secret = true;
     return true;
 }
 
 static bool
-g23_parse_seckey(pgp_key_pkt_t &seckey,
-                 const uint8_t *data,
-                 size_t         data_len,
-                 const char *   password)
+g23_parse_seckey(pgp_key_pkt_t &  seckey,
+                 const uint8_t *  data,
+                 size_t           data_len,
+                 const char *     password,
+                 pgp_pubkey_alg_t pubalg = PGP_PKA_NOTHING)
 {
     gnupg_extended_private_key_t g23_extended_key;
 
-    const char *bytes = (const char *) data;
-    if (!g23_extended_key.parse(bytes, data_len, SXP_MAX_DEPTH)) {
+    if (!g23_extended_key.parse((const char *) data, data_len, SXP_MAX_DEPTH)) {
         RNP_LOG("Failed to parse s-exp.");
         return false;
     }
@@ -804,19 +827,22 @@ g23_parse_seckey(pgp_key_pkt_t &seckey,
         return false;
     }
 
-    auto &           alg_bt = alg_s_exp->sexp_string_at(0)->get_string();
-    pgp_pubkey_alg_t alg = static_cast<pgp_pubkey_alg_t>(
-      id_str_pair::lookup(g10_alg_aliases, alg_bt.c_str(), PGP_PKA_NOTHING));
-    if (alg == PGP_PKA_NOTHING) {
-        RNP_LOG(
-          "Unsupported algorithm: '%.*s'", (int) alg_bt.size(), (const char *) alg_bt.data());
-        return false;
-    }
-
     bool ret = false;
-    if (!parse_pubkey(seckey, alg_s_exp, alg)) {
-        RNP_LOG("failed to parse pubkey");
-        goto done;
+    auto alg = pubalg;
+    if (alg == PGP_PKA_NOTHING) {
+        auto &alg_bt = alg_s_exp->sexp_string_at(0)->get_string();
+        auto  alg_st = (const char *) alg_bt.data();
+        alg = static_cast<pgp_pubkey_alg_t>(
+          id_str_pair::lookup(g10_alg_aliases, alg_st, PGP_PKA_NOTHING));
+        if (alg == PGP_PKA_NOTHING) {
+            RNP_LOG("Unsupported algorithm: '%.*s'", (int) alg_bt.size(), alg_st);
+            return false;
+        }
+        /* Parse pubkey only if it was not parsed before */
+        if (!parse_pubkey(seckey, alg_s_exp, alg)) {
+            RNP_LOG("failed to parse pubkey");
+            goto done;
+        }
     }
 
     if (is_protected) {
@@ -827,7 +853,7 @@ g23_parse_seckey(pgp_key_pkt_t &seckey,
         seckey.sec_protection.s2k.usage = PGP_S2KU_NONE;
         seckey.sec_protection.symm_alg = PGP_SA_PLAINTEXT;
         seckey.sec_protection.s2k.hash_alg = PGP_HASH_UNKNOWN;
-        if (!parse_seckey(seckey, alg_s_exp, alg)) {
+        if (!parse_seckey(seckey, alg_s_exp)) {
             RNP_LOG("failed to parse seckey");
             goto done;
         }
@@ -841,21 +867,18 @@ done:
 }
 
 pgp_key_pkt_t *
-g10_decrypt_seckey(const pgp_rawpacket_t &raw,
-                   const pgp_key_pkt_t &  pubkey,
-                   const char *           password)
+g10_decrypt_seckey(const rnp::RawPacket &raw,
+                   const pgp_key_pkt_t & pubkey,
+                   const char *          password)
 {
     if (!password) {
         return NULL;
     }
     auto seckey = std::unique_ptr<pgp_key_pkt_t>(new pgp_key_pkt_t(pubkey, false));
-    if (!g23_parse_seckey(*seckey, raw.raw.data(), raw.raw.size(), password)) {
-        return NULL;
+    if (!g23_parse_seckey(
+          *seckey, raw.data().data(), raw.data().size(), password, pubkey.alg)) {
+        return nullptr;
     }
-    /* g10 has the same 'ecc' algo for ECDSA/ECDH/EDDSA. Probably should be better place to fix
-     * this. */
-    seckey->alg = pubkey.alg;
-    seckey->material.alg = pubkey.material.alg;
     return seckey.release();
 }
 
@@ -864,64 +887,72 @@ copy_secret_fields(pgp_key_pkt_t &dst, const pgp_key_pkt_t &src)
 {
     switch (src.alg) {
     case PGP_PKA_DSA:
-        dst.material.dsa.x = src.material.dsa.x;
+        if (src.material->secret()) {
+            auto &dsasrc = dynamic_cast<pgp::DSAKeyMaterial &>(*src.material);
+            auto &dsadst = dynamic_cast<pgp::DSAKeyMaterial &>(*dst.material);
+            dsadst.set_secret(dsasrc.x());
+        }
         break;
     case PGP_PKA_RSA:
     case PGP_PKA_RSA_ENCRYPT_ONLY:
     case PGP_PKA_RSA_SIGN_ONLY:
-        dst.material.rsa.d = src.material.rsa.d;
-        dst.material.rsa.p = src.material.rsa.p;
-        dst.material.rsa.q = src.material.rsa.q;
-        dst.material.rsa.u = src.material.rsa.u;
+        if (src.material->secret()) {
+            auto &rsasrc = dynamic_cast<pgp::RSAKeyMaterial &>(*src.material);
+            auto &rsadst = dynamic_cast<pgp::RSAKeyMaterial &>(*dst.material);
+            rsadst.set_secret(rsasrc.d(), rsasrc.p(), rsasrc.q(), rsasrc.u());
+        }
         break;
     case PGP_PKA_ELGAMAL:
     case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN:
-        dst.material.eg.x = src.material.eg.x;
+        if (src.material->secret()) {
+            auto &egsrc = dynamic_cast<pgp::EGKeyMaterial &>(*src.material);
+            auto &egdst = dynamic_cast<pgp::EGKeyMaterial &>(*dst.material);
+            egdst.set_secret(egsrc.x());
+        }
         break;
     case PGP_PKA_ECDSA:
     case PGP_PKA_ECDH:
     case PGP_PKA_EDDSA:
-        dst.material.ec.x = src.material.ec.x;
+        if (src.material->secret()) {
+            auto &ecsrc = dynamic_cast<pgp::ECKeyMaterial &>(*src.material);
+            auto &ecdst = dynamic_cast<pgp::ECKeyMaterial &>(*dst.material);
+            ecdst.set_secret(ecsrc.x());
+        }
         break;
     default:
         RNP_LOG("Unsupported public key algorithm: %d", (int) src.alg);
         return false;
     }
 
-    dst.material.secret = src.material.secret;
     dst.sec_protection = src.sec_protection;
     dst.tag = is_subkey_pkt(dst.tag) ? PGP_PKT_SECRET_SUBKEY : PGP_PKT_SECRET_KEY;
     return true;
 }
 
+namespace rnp {
 bool
-rnp_key_store_g10_from_src(rnp_key_store_t *         key_store,
-                           pgp_source_t *            src,
-                           const pgp_key_provider_t *key_provider)
+KeyStore::load_g10(pgp_source_t &src, const KeyProvider *key_provider)
 {
     try {
         /* read src to the memory */
-        rnp::MemorySource memsrc(*src);
+        MemorySource memsrc(src);
         /* parse secret key: fills material and sec_protection only */
         pgp_key_pkt_t seckey;
         if (!g23_parse_seckey(seckey, (uint8_t *) memsrc.memory(), memsrc.size(), NULL)) {
             return false;
         }
         /* copy public key fields if any */
-        pgp_key_t key;
+        Key key;
         if (key_provider) {
-            pgp_key_request_ctx_t req_ctx(PGP_OP_MERGE_INFO, false, PGP_KEY_SEARCH_GRIP);
-            if (!rnp_key_store_get_key_grip(&seckey.material, req_ctx.search.by.grip)) {
-                return false;
-            }
-
-            const pgp_key_t *pubkey = pgp_request_key(key_provider, &req_ctx);
+            pgp::KeyGrip grip = seckey.material->grip();
+            auto         pubkey =
+              key_provider->request_key(*KeySearch::create(grip), PGP_OP_MERGE_INFO);
             if (!pubkey) {
                 return false;
             }
 
             /* public key packet has some more info then the secret part */
-            key = pgp_key_t(*pubkey, true);
+            key = Key(*pubkey, true);
             if (!copy_secret_fields(key.pkt(), seckey)) {
                 return false;
             }
@@ -930,17 +961,20 @@ rnp_key_store_g10_from_src(rnp_key_store_t *         key_store,
         }
         /* set rawpkt */
         key.set_rawpkt(
-          pgp_rawpacket_t((uint8_t *) memsrc.memory(), memsrc.size(), PGP_PKT_RESERVED));
-        key.format = PGP_KEY_STORE_G10;
-        if (!rnp_key_store_add_key(key_store, &key)) {
+          RawPacket((uint8_t *) memsrc.memory(), memsrc.size(), PGP_PKT_RESERVED));
+        key.format = KeyFormat::G10;
+        if (!add_key(key)) {
             return false;
         }
         return true;
     } catch (const std::exception &e) {
+        /* LCOV_EXCL_START */
         RNP_LOG("%s", e.what());
         return false;
+        /* LCOV_EXCL_END */
     }
 }
+} // namespace rnp
 
 /*
  * Write G10 S-exp to buffer
@@ -970,33 +1004,41 @@ void
 gnupg_sexp_t::add_pubkey(const pgp_key_pkt_t &key)
 {
     switch (key.alg) {
-    case PGP_PKA_DSA:
+    case PGP_PKA_DSA: {
+        auto &dsa = dynamic_cast<const pgp::DSAKeyMaterial &>(*key.material);
         add("dsa");
-        add_mpi("p", key.material.dsa.p);
-        add_mpi("q", key.material.dsa.q);
-        add_mpi("g", key.material.dsa.g);
-        add_mpi("y", key.material.dsa.y);
+        add_mpi("p", dsa.p());
+        add_mpi("q", dsa.q());
+        add_mpi("g", dsa.g());
+        add_mpi("y", dsa.y());
         break;
+    }
     case PGP_PKA_RSA_SIGN_ONLY:
     case PGP_PKA_RSA_ENCRYPT_ONLY:
-    case PGP_PKA_RSA:
+    case PGP_PKA_RSA: {
+        auto &rsa = dynamic_cast<const pgp::RSAKeyMaterial &>(*key.material);
         add("rsa");
-        add_mpi("n", key.material.rsa.n);
-        add_mpi("e", key.material.rsa.e);
+        add_mpi("n", rsa.n());
+        add_mpi("e", rsa.e());
         break;
-    case PGP_PKA_ELGAMAL:
+    }
+    case PGP_PKA_ELGAMAL: {
+        auto &eg = dynamic_cast<const pgp::EGKeyMaterial &>(*key.material);
         add("elg");
-        add_mpi("p", key.material.eg.p);
-        add_mpi("g", key.material.eg.g);
-        add_mpi("y", key.material.eg.y);
+        add_mpi("p", eg.p());
+        add_mpi("g", eg.g());
+        add_mpi("y", eg.y());
         break;
+    }
     case PGP_PKA_ECDSA:
     case PGP_PKA_ECDH:
-    case PGP_PKA_EDDSA:
+    case PGP_PKA_EDDSA: {
+        auto &ec = dynamic_cast<const pgp::ECKeyMaterial &>(*key.material);
         add("ecc");
-        add_curve("curve", key.material.ec);
-        add_mpi("q", key.material.ec.p);
+        add_curve("curve", ec.curve());
+        add_mpi("q", ec.p());
         break;
+    }
     default:
         RNP_LOG("Unsupported public key algorithm: %d", (int) key.alg);
         throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
@@ -1007,24 +1049,31 @@ void
 gnupg_sexp_t::add_seckey(const pgp_key_pkt_t &key)
 {
     switch (key.alg) {
-    case PGP_PKA_DSA:
-        add_mpi("x", key.material.dsa.x);
+    case PGP_PKA_DSA: {
+        auto &dsa = dynamic_cast<const pgp::DSAKeyMaterial &>(*key.material);
+        add_mpi("x", dsa.x());
         break;
+    }
     case PGP_PKA_RSA_SIGN_ONLY:
     case PGP_PKA_RSA_ENCRYPT_ONLY:
-    case PGP_PKA_RSA:
-        add_mpi("d", key.material.rsa.d);
-        add_mpi("p", key.material.rsa.p);
-        add_mpi("q", key.material.rsa.q);
-        add_mpi("u", key.material.rsa.u);
+    case PGP_PKA_RSA: {
+        auto &rsa = dynamic_cast<const pgp::RSAKeyMaterial &>(*key.material);
+        add_mpi("d", rsa.d());
+        add_mpi("p", rsa.p());
+        add_mpi("q", rsa.q());
+        add_mpi("u", rsa.u());
         break;
-    case PGP_PKA_ELGAMAL:
-        add_mpi("x", key.material.eg.x);
+    }
+    case PGP_PKA_ELGAMAL: {
+        auto &eg = dynamic_cast<const pgp::EGKeyMaterial &>(*key.material);
+        add_mpi("x", eg.x());
         break;
+    }
     case PGP_PKA_ECDSA:
     case PGP_PKA_ECDH:
     case PGP_PKA_EDDSA: {
-        add_mpi("d", key.material.ec.x);
+        auto &ec = dynamic_cast<const pgp::ECKeyMaterial &>(*key.material);
+        add_mpi("d", ec.x());
         break;
     }
     default:
@@ -1033,7 +1082,7 @@ gnupg_sexp_t::add_seckey(const pgp_key_pkt_t &key)
     }
 }
 
-rnp::secure_vector<uint8_t>
+rnp::secure_bytes
 gnupg_sexp_t::write_padded(size_t padblock) const
 {
     rnp::MemoryDest raw;
@@ -1054,7 +1103,7 @@ gnupg_sexp_t::write_padded(size_t padblock) const
         throw rnp::rnp_exception(RNP_ERROR_BAD_STATE);
     }
     const uint8_t *mem = (uint8_t *) raw.memory();
-    return rnp::secure_vector<uint8_t>(mem, mem + raw.writeb());
+    return rnp::secure_bytes(mem, mem + raw.writeb());
 }
 
 void
@@ -1100,7 +1149,7 @@ gnupg_sexp_t::add_protected_seckey(pgp_key_pkt_t &       seckey,
     psub_s_exp->add(checksum, sizeof(checksum));
 
     /* write raw secret key to the memory */
-    rnp::secure_vector<uint8_t> rawkey = raw_s_exp.write_padded(format->cipher_block_size);
+    rnp::secure_bytes rawkey = raw_s_exp.write_padded(format->cipher_block_size);
 
     /* derive encrypting key */
     unsigned keysize = pgp_key_size(prot.symm_alg);
@@ -1231,13 +1280,12 @@ g10_calculated_hash(const pgp_key_pkt_t &key, const char *protected_at, uint8_t
 }
 
 bool
-rnp_key_store_gnupg_sexp_to_dst(pgp_key_t *key, pgp_dest_t *dest)
+rnp_key_store_gnupg_sexp_to_dst(rnp::Key &key, pgp_dest_t &dest)
 {
-    if (key->format != PGP_KEY_STORE_G10) {
-        RNP_LOG("incorrect format: %d", key->format);
+    if (key.format != rnp::KeyFormat::G10) {
+        RNP_LOG("incorrect format: %d", static_cast<int>(key.format));
         return false;
     }
-    pgp_rawpacket_t &packet = key->rawpkt();
-    dst_write(dest, packet.raw.data(), packet.raw.size());
-    return dest->werr == RNP_SUCCESS;
+    dst_write(dest, key.rawpkt().data());
+    return dest.werr == RNP_SUCCESS;
 }
