@@ -10,10 +10,11 @@
 #include "../rnp/fficli.h"
 #include "logging.h"
 
-#define DEFAULT_RSA_NUMBITS 2048
+#define DEFAULT_RSA_NUMBITS 3072
 
 typedef enum {
     /* commands */
+    CMD_NONE = 0,
     CMD_LIST_KEYS = 260,
     CMD_EXPORT_KEY,
     CMD_IMPORT,
@@ -32,6 +33,8 @@ typedef enum {
     OPT_USERID,
     OPT_HOMEDIR,
     OPT_NUMBITS,
+    OPT_ALLOW_WEAK_HASH,
+    OPT_ALLOW_SHA1,
     OPT_HASH_ALG,
     OPT_COREDUMPS,
     OPT_PASSWDFD,
@@ -54,6 +57,10 @@ typedef enum {
     OPT_FIX_25519_BITS,
     OPT_CHK_25519_BITS,
     OPT_CURTIME,
+    OPT_ALLOW_OLD_CIPHERS,
+    OPT_ADD_SUBKEY,
+    OPT_SET_EXPIRE,
+    OPT_KEYFILE,
 
     /* debug */
     OPT_DEBUG
@@ -71,6 +78,6 @@ void print_usage(const char *usagemsg);
  * @param cfg configuration with settings from command line
  * @return true on success, or false otherwise.
  */
-bool rnpkeys_init(cli_rnp_t *rnp, const rnp_cfg &cfg);
+bool rnpkeys_init(cli_rnp_t &rnp, const rnp_cfg &cfg);
 
 #endif /* _rnpkeys_ */
